@@ -1,11 +1,12 @@
 package Crypt::SecretBuffer;
 # VERSION
 # ABSTRACT: Prevent accidentally leaking a string of sensitive data
-$Crypt::SecretBuffer::VERSION = '0.016';
+$Crypt::SecretBuffer::VERSION = '0.017';
 
 use strict;
 use warnings;
 use Carp;
+use IO::Handle;
 use Scalar::Util ();
 use parent qw( DynaLoader );
 use overload '""' => \&stringify,
@@ -17,7 +18,7 @@ bootstrap Crypt::SecretBuffer;
 
 {
    package Crypt::SecretBuffer::Exports;
-$Crypt::SecretBuffer::Exports::VERSION = '0.016';
+$Crypt::SecretBuffer::Exports::VERSION = '0.017';
 use Exporter 'import';
    @Crypt::SecretBuffer::Exports::EXPORT_OK= qw(
       secret_buffer secret unmask_secrets_to memcmp
@@ -62,17 +63,95 @@ sub stringify_mask {
 
 sub append_console_line {
    my ($self, $handle, %options)= @_;
-   my $echo_off= Crypt::SecretBuffer::ConsoleState->maybe_new(
+   my ($prompt, $prompt_fh, $char_mask, $char_count, $char_max, $char_class)
+      = delete @options{qw( prompt prompt_fh char_mask char_count char_max char_class )};
+   warn "unknown option: ".join(', ', keys %options)
+      if keys %options;
+   if (!$prompt_fh && (defined $prompt || defined $char_mask)) {
+      my $fd= fileno($handle);
+      if (defined $fd && $fd >= 0) {
+         $prompt_fh= IO::Handle->new_from_fd($fd, 'w');
+      } elsif ($handle == \*STDIN) {
+         $prompt_fh= \*STDOUT;
+      } else {
+         $prompt_fh= $handle;
+      }
+   }
+   # If the user wants control over the keypresses, need to disable line-editing mode
+   my $input_by_chars= defined $char_mask || defined $char_count || defined $char_class;
+   my $ttystate= Crypt::SecretBuffer::ConsoleState->maybe_new(
       handle => $handle,
       echo => 0,
+      (line_input => 0)x!!$input_by_chars,
       auto_restore => 1
    );
-   if (defined(my $prompt= delete $options{prompt})) {
-      my $prompt_fh= delete $options{prompt_fh} || $handle;
+   if (defined $prompt) {
       $prompt_fh->print($prompt);
       $prompt_fh->flush;
    }
-   return $self->_append_console_line($handle);
+   my $start_len= $self->length;
+   if ($input_by_chars) {
+      while (1) {
+         $self->append_read($handle, 1)
+            or return undef;
+         # Handle control characters
+         my $end_pos= $self->length - 1;
+         if ($self->index(qr/[\0-\x1F\x7F]/, $end_pos) == $end_pos) {
+            # If it is \r or \n, end.  If char_count was requested, and we didn't
+            # end by that logic, then we don't have the requested char count, so
+            # return false.
+            if ($self->index(qr/[\r\n]/, $end_pos) == $end_pos) {
+               $self->length($end_pos); # remove CR or LF
+               return !$char_count;
+            }
+            # handle backspace
+            elsif ($self->index(qr/[\b\x7F]/, $end_pos) == $end_pos) {
+               $self->length($end_pos); # remove backspace
+               if ($self->length > $start_len) {
+                  $self->length($self->length-1); # remove previous char
+                  # print a backspace + space + backspace to erase the final mask character
+                  if (length $char_mask) {
+                     $prompt_fh->print(
+                        ("\b" x length $char_mask)
+                       .(" "  x length $char_mask)
+                       .("\b" x length $char_mask));
+                     $prompt_fh->flush;
+                  }
+               }
+            }
+            # just ignore any other control char
+            else {
+               $self->length($end_pos);
+            }
+         }
+         elsif ($char_class && $self->index($char_class, $end_pos) == -1) {
+            # not part of the permitted char class
+            $self->length($end_pos);
+         }
+         elsif ($char_max && $self->length - $start_len > $char_max) {
+            # refuse to add more characters
+            $self->length($end_pos);
+         }
+         else {
+            # char added
+            if (length $char_mask) {
+               $prompt_fh->print($char_mask);
+               $prompt_fh->flush;
+            }
+            # If reached the char_count, return success
+            return 1
+               if $char_count && $self->length - $start_len >= $char_count;
+         }
+      }
+   }
+   else {
+      my $ret= $self->_append_console_line($handle);
+      if ($char_max && $self->length - $start_len > $char_max) {
+         # truncate the input if char_max requested
+         $self->length($start_len + $char_max);
+      }
+      return $ret;
+   }
 }
 
 
@@ -481,8 +560,11 @@ of bytes and never blocks.
 
   $bool= $buf->append_console_line($handle);
   $bool= $buf->append_console_line($handle,
-    prompt => "Enter Password: ",
-    prompt_fh => $alternate_handle,   # optional
+    prompt     => "Enter Password: ", # print prompt after disable echo
+    prompt_fh  => $alternate_handle,  # optional, handle for writing prompt
+    char_mask  => "*",                # show each char typed as '*'
+    char_count => $n,                 # stop after N characters (or newline)
+    char_class => qr/[...]/,          # limit to members of character class
   );
 
 This turns off TTY echo (if the handle is a Unix TTY or Windows Console) and reads and appends
@@ -495,9 +577,66 @@ When possible, this reads directly from the OS to avoid buffering the secret in
 but reads from the buffer if you already have input data in one of those buffers, or if the
 file handle is a virtual Perl handle not backed by the OS.
 
-If you specify a prompt (new in version 0.016), the TTY echo is disabled before printing the
-prompt.  This helps prevent a race condition where a scripted interaction could start typing a
-password in response to the prompt before the echo was disabled.
+Options:
+
+=over
+
+=item prompt
+
+This message is printed and flushed after disabling TTY echo.  This helps prevent a race
+condition where a scripted interaction could start typing a password in response to the prompt
+before the echo was disabled.
+
+=item prompt_fh
+
+If C<$handle> is STDIN (the most common scenario) perl will not be able to write to it because
+libc opened it with 'r' mode, even though the TTY is always writeable.  The automatic workaround
+is to get the file descriptor and open a second handle to it with 'w' mode.  If the automatic
+behavior is wrong for your use case, you can provide the write-handle with this option.
+
+=item char_mask
+
+Display this static string every time the user types a key, for feedback.  A common choice would
+be C<'*'> or C<'* '>.
+
+=item char_count
+
+Stop after N characters have been added to the buffer.  Note that unicode is not supported yet,
+so this really means N bytes.  Reaching N bytes is considered a success and causes
+append_console_line to return true.  If the user presses newline before N bytes,
+append_console_line will return false.
+
+=item char_max
+
+Stop appending characters when N have been added to the buffer, but don't return until the user
+presses newline.
+
+=item char_class
+
+Restrict the permitted characters.  This must be a Regexp-ref of a single character class.
+Any character the user enters which is not in this class will be ignored and not added to the
+buffer.
+
+=back
+
+When using options C<char_mask>, C<char_count>, or C<char_class>, the TTY line-input mode is
+disabled and the code processes each character as it is received, manually handling backspace
+etc.  The code does I<not> handle TTY geometry or unicode, and will display incorrectly if the
+user's input reaches the edge of the terminal.  This won't usually be a problem if you just
+want some fancy handling of N-digit codes where you want to return as soon as they reach the
+limit:
+
+  $buf->append_console_line(STDIN,
+    prompt => "PIN: [             ]\b\b\b\b\b\b\b\b\b\b\b\b\b",
+    char_mask  => "* ",
+    char_count => 6,
+    char_class => qr/[0-9]/,
+  );
+
+If this method doesn't have quite the behavior you were looking for, the read loop is perl
+(not XS) and the cross-platform handling of console modes happens in
+L<Crypt::SecretBuffer::ConsoleState>, so it should be reasonably easy to copy/paste and make
+your own.
 
 =head2 append_sysread
 
@@ -736,7 +875,7 @@ instructions how to report security vulnerabilities.
 
 =head1 VERSION
 
-version 0.016
+version 0.017
 
 =head1 AUTHOR
 
