@@ -224,7 +224,8 @@ static const char base64_alphabet[64]=
    "0123456789+/";
 
 static const int8_t base64_decode_table[256]= {
-   [0 ... 255] = -1,
+   [0 ... 42] = -1, [44 ... 46] = -1, [58 ... 60] = -1,
+   [62 ... 64] = -1, [91 ... 96] = -1, [123 ... 255] = -1,
    /* A–Z = 0–25 */
    ['A'] = 0,  ['B'] = 1,  ['C'] = 2,  ['D'] = 3,
    ['E'] = 4,  ['F'] = 5,  ['G'] = 6,  ['H'] = 7,
